package io.redvox.api900;

import java.util.*;


public class Util {
    public static <T> Map<T, T> listToKeyPair(final List<T> list) {
        if(list.size() % 2 == 1) {
            throw Api900Exception.of("List must contain even number of items");
        }

        final Map<T, T> keyPairs = new HashMap<>();
        final ListIterator<T> iterator = list.listIterator();
        while(iterator.hasNext()) {
            keyPairs.put(iterator.next(), iterator.next());
        }
        return keyPairs;
    }

    public static <T extends Number> double mean(final List<T> vs) {
        if(vs.size() == 0) {
            throw Api900Exception.of("mean requires at least 1 sample point");
        }
        double sum = 0;
        for(final T v: vs) {
            sum += v.doubleValue();
        }
        return sum / vs.size();
    }

    public static <T extends Number & Comparable<T>> double median(final List<T> vs) {
        if(vs.size() == 0) {
            throw Api900Exception.of("median requires at least 1 sample point");
        }
        Collections.sort(vs);
        final int middle = vs.size() / 2;
        if (vs.size() % 2 == 1) {
            return vs.get(middle).doubleValue();
        } else {
            return (vs.get(middle-1).doubleValue() + vs.get(middle).doubleValue()) / 2.0;
        }
    }

    public static <T extends Number> double variance(final List<T> vs) {
        if(vs.size() < 2) {
            throw Api900Exception.of("variance requires at least 2 sample points");
        }
        final double mean = mean(vs);
        double sqSum = 0;
        double d;
        for(final T v: vs) {
            d = v.doubleValue();
            sqSum += (d - mean) * (d - mean);
        }
        return sqSum / vs.size();
    }

    public static <T extends Number> double stddev(final List<T> vs) {
        if(vs.size() < 2) {
            throw Api900Exception.of("stddev requires at least 2 sample points");
        }
        return Math.sqrt(variance(vs));
    }
}
